"""Approvaltests.scrubbers module."""

from .scrubbers import (
    combine_scrubbers,
    create_regex_scrubber,
    scrub_all_dates,
    scrub_all_guids,
    templates_regex_scrubber_with_lambda,
    templates_regex_scrubber_with_replacement,
)
